-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "FA_AddPoint"

-- **************************************************
-- General information about this script
-- **************************************************

FA_AddPoint = {}

function FA_AddPoint:Name()
	return "Add Point"
end

function FA_AddPoint:Version()
	return "5.0"
end

function FA_AddPoint:Description()
	return "Add a point (<shift> to add selection, <space> to weld, hold <alt> to disable auto-welding)"
end

function FA_AddPoint:Creator()
	return "Lost Marble, Modified by Fazek"
end

function FA_AddPoint:UILabel()
	return("Add Point")
end

function FA_AddPoint:LoadPrefs(prefs)
	FA_SharedUtils:Load_Prefs(prefs)
end

function FA_AddPoint:SavePrefs(prefs)
	FA_SharedUtils:Save_Prefs(prefs)
end

-- **************************************************
-- Recurring values
-- **************************************************

FA_AddPoint.dragging = false
FA_AddPoint.delayed= false
FA_AddPoint.numSel = 0
FA_AddPoint.selID = 0

-- **************************************************
-- The guts of this script
-- **************************************************

function FA_AddPoint:IsEnabled(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return false
	end
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	return true
end

function FA_AddPoint:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then return end

	if (mouseEvent.shiftKey and not mouseEvent.altKey) then
--force selection
		FA_SharedUtils.selectionMode= 0
		FA_SharedUtils:MouseDown(moho, mouseEvent)
		self.dragging= false
		return
	end
	FA_SharedUtils.selectionMode= -1

	self.dragging= true
	self.delayed= false
	self.selID= -1
	local pt

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	local closestID= mouseEvent.view:PickPoint(mouseEvent.pt)

	if (closestID >= 0) then
		pt= mesh:Point(closestID)
		pt.fSelected= not pt.fSelected
		self.delayed= true
		self.selID= closestID
		mouseEvent.view:DrawMe()
		return
	end

	if (not mouseEvent.altKey) then
-- auto-welding?
		local testVec1 = LM.Vector2:new_local()
		testVec1:Set(mouseEvent.vec)

		if (closestID < 0) then
			closestID = mesh:ClosestPoint(testVec1)
		end

		if (closestID >= 0) then
			local testVec2 = LM.Vector2:new_local()
			testVec2:Set(mesh:Point(closestID).fPos)
	-- convert the two chosen points to pixel coordinates
	-- this is to make sure that they are actually close enough in screen space

			local p1 = LM.Point:new_local()
			local p2 = LM.Point:new_local()
			local m = LM.Matrix:new_local()
			moho.layer:GetFullTransform(moho.frame, m, moho.document)
			m:Transform(testVec1)
			m:Transform(testVec2)
			mouseEvent.view:Graphics():WorldToScreen(testVec1, p1)
			mouseEvent.view:Graphics():WorldToScreen(testVec2, p2)
			p1.x = p1.x - p2.x
			p1.y = p1.y - p2.y

			if ((p1.x * p1.x) + (p1.y * p1.y) < 100) then

				if (mesh:Point(closestID):IsEndpoint()) then
		-- extend this curve
					mesh:AddPoint(mouseEvent.vec, closestID, moho.frame)
				else
		-- start a new curve that is connected to this point
					mesh:AddPoint(mouseEvent.vec, -1, moho.frame)

					if (FA_SharedUtils.sharpCorners) then
						mesh:Point(closestID):SetCurvature(MOHO.PEAKED, 0)
					end
					mesh:WeldPoints(mesh:CountPoints() - 2, closestID, moho.frame)
				end
				self.selID = mesh:CountPoints() - 1

				if (FA_SharedUtils.sharpCorners) then
					mesh:Point(self.selID):SetCurvature(MOHO.PEAKED, 0)
				end
			end
		end
	end

	if (self.selID < 0) then
--clicked on edge?
		local curveID = -1
		local segID = -1
		local sharp= FA_SharedUtils.sharpCorners
		local selected= moho:CountSelectedPoints()

		if (not mouseEvent.altKey) then
			curveID, segID = mouseEvent.view:PickEdge(mouseEvent.pt, curveID, segID)
		end

		if (curveID >= 0) then
-- add a point in the middle of some curve

			if (moho.gridOn) then
				moho:SnapToGrid(mouseEvent.vec)
			end
			mesh:AddPoint(mouseEvent.vec, curveID, segID, moho.frame)
			closestID= mesh:CountPoints() - 1
			selected= 3
		elseif (selected > 0) then
-- link to all selected points

			closestID= mesh:CountPoints()

			if (selected > 3) then
				sharp= true
			end

			for i= 0, closestID - 1 do
				pt= mesh:Point(i)

				if (pt.fSelected) then
					local ptN= mesh:CountPoints()
					mesh:AddLonePoint(mouseEvent.vec, moho.frame)
					mesh:AppendPoint(pt.fPos, moho.frame)

					if (sharp) then
						mesh:Point(mesh:CountPoints() - 1):SetCurvature(MOHO.PEAKED, 0)

						if (pt:IsEndpoint()) then
							pt:SetCurvature(MOHO.PEAKED, 0)
						end
					end

					if (ptN ~= closestID) then
						mesh:WeldPoints(ptN, closestID, moho.frame)
					end
					mesh:WeldPoints(mesh:CountPoints() - 1, i, moho.frame)
				end
			end
		else
-- add a new curve segment
			if (moho.gridOn) then
				moho:SnapToGrid(mouseEvent.vec)
			end
			mesh:AddPoint(mouseEvent.vec, -1, moho.frame)

			if (sharp) then
				mesh:Point(mesh:CountPoints() - 2):SetCurvature(MOHO.PEAKED, 0)
			end
			closestID= mesh:CountPoints() - 1
			selected= 3
		end

		if (sharp) then
			mesh:Point(closestID):SetCurvature(MOHO.PEAKED, 0)
		end

		if (selected < 3 and FA_SharedUtils.autoFill) then
			FA_SharedUtils:CreateShape(moho, mesh, closestID)
		end
		self.selID= closestID
	end
	mesh:SelectNone()
	pt= mesh:Point(self.selID)
	pt.fSelected= true
	pt.fTempPos:Set(pt.fPos)
	mouseEvent.view:DrawMe()
end

function FA_AddPoint:OnMouseMoved(moho, mouseEvent)

	if (FA_SharedUtils.selectionMode >= 0) then
		FA_SharedUtils:MouseMoved(moho,mouseEvent)
		return
	end

	if (not self.dragging) then return end

	local mesh = moho:Mesh()
	if (mesh == nil) then return end

	if (self.delayed) then
--moved?
		local x= mouseEvent.startPt.x - mouseEvent.pt.x
		local y= mouseEvent.startPt.y - mouseEvent.pt.y

		if ((x*x + y*y) < 9) then return end

		self.delayed= false

		if (mesh:Point(self.selID):IsEndpoint()) then
	-- extend this curve
			mesh:AddPoint(mouseEvent.startVec, self.selID, moho.frame)
		else
	-- start a new curve that is connected to this point
			mesh:AddPoint(mouseEvent.startVec, -1, moho.frame)

			if (FA_SharedUtils.sharpCorners) then
				mesh:Point(self.selID):SetCurvature(MOHO.PEAKED, 0)
			end
			mesh:WeldPoints(mesh:CountPoints() - 2, self.selID, moho.frame)
		end
		self.selID = mesh:CountPoints() - 1

		if (FA_SharedUtils.sharpCorners) then
			mesh:Point(self.selID):SetCurvature(MOHO.PEAKED, 0)
		end
		mesh:SelectNone()
		local pt= mesh:Point(self.selID)
		pt.fSelected= true
		pt.fTempPos:Set(pt.fPos)
	end
	local curVec = mouseEvent.vec - mouseEvent.startVec

	if (mouseEvent.ctrlKey) then
		if (math.abs(mouseEvent.vec.x - mouseEvent.startVec.x) > math.abs(mouseEvent.vec.y - mouseEvent.startVec.y)) then
			curVec.y = 0
		else
			curVec.x = 0
		end
	end

	if (self.selID >= 0) then
		local pt= mesh:Point(self.selID)
		pt.fPos:Set(pt.fTempPos + curVec)

		if (moho.gridOn) then
			moho:SnapToGrid(pt.fPos)
		end
	end
	moho:AddPointKeyframe(moho.frame)
	mouseEvent.view:DrawMe()
end

function FA_AddPoint:OnMouseUp(moho, mouseEvent)

	if (FA_SharedUtils.selectionMode >= 0) then
		FA_SharedUtils:MouseUp(moho,mouseEvent)
		return
	end

	if (not self.dragging) then return end

	self.dragging= false

	local mesh = moho:Mesh()
	if (mesh == nil) then return end

	if (self.delayed) then
		self.selID= -1
		return
	end

	if (mesh:Point(self.selID):IsEndpoint()) then
		FA_SharedUtils:TryWeldPoints(moho,mouseEvent.view,true,self.selID)
	end
	moho:AddPointKeyframe(moho.frame)
	moho:NewKeyframe(CHANNEL_POINT)

	self.selID= -1
	mouseEvent.view:DrawMe()
end

function FA_AddPoint:OnKeyDown(moho, keyEvent)

	if (keyEvent.key == ' ') then
		local doit = false

		if (self.dragging) then
			if (self.numSel == 1) then
				doit = true
			end
		elseif (moho:CountSelectedPoints() == 1) then
			doit = true
		end

		if (doit) then
-- try welding the selected point to the closest other point

			for i = 0, mesh:CountPoints() - 1 do

				if (mesh:Point(i).fSelected) then
					FA_SharedUtils:TryWeldPoints(moho,keyEvent.view,self.dragging,i)
					self.selID= -1
					break
				end
			end
			keyEvent.view:DrawMe()
		end

	elseif (keyEvent.ctrlKey) then
		local inc = 1

		if (keyEvent.shiftKey) then
			inc = 10
		end
		local m = LM.Matrix:new_local()
		moho.layer:GetFullTransform(moho.frame, m, moho.document)

		if (keyEvent.keyCode == LM.GUI.KEY_UP) then
			FA_SharedUtils:TranslateStep(moho,mesh,keyEvent.view,0,-inc)
		elseif (keyEvent.keyCode == LM.GUI.KEY_DOWN) then
			FA_SharedUtils:TranslateStep(moho,mesh,keyEvent.view,0,inc)
		elseif (keyEvent.keyCode == LM.GUI.KEY_LEFT) then
			FA_SharedUtils:TranslateStep(moho,mesh,keyEvent.view,-inc,0)
		elseif (keyEvent.keyCode == LM.GUI.KEY_RIGHT) then
			FA_SharedUtils:TranslateStep(moho,mesh,keyEvent.view,inc,0)
		end
	else
		FA_SharedUtils:KeyDown(moho,keyEvent)
	end
end

function FA_AddPoint:DrawMe(moho, view)
	FA_SharedUtils:Draw_Me(moho, view, -1)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

FA_AddPoint.DUMMY = MOHO.MSG_BASE
FA_AddPoint.AUTOWELD = MOHO.MSG_BASE + 1
FA_AddPoint.AUTOFILL = MOHO.MSG_BASE + 2
FA_AddPoint.SHARPCORNERS = MOHO.MSG_BASE + 3
FA_AddPoint.DELETE = MOHO.MSG_BASE + 4
FA_AddPoint.SELECTITEM = MOHO.MSG_BASE + 5

function FA_AddPoint:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu(FA_SharedUtils.txSelectGroup)

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	layout:AddChild(LM.GUI.Button(FA_SharedUtils.txDelete, self.DELETE))

	self.autoWeldCheck = LM.GUI.CheckBox(FA_SharedUtils.txAutoWeld, self.AUTOWELD)
	layout:AddChild(self.autoWeldCheck)

	self.autoFillCheck = LM.GUI.CheckBox(FA_SharedUtils.txAutoFill, self.AUTOFILL)
	layout:AddChild(self.autoFillCheck)

	self.sharpCornersCheck = LM.GUI.CheckBox(FA_SharedUtils.txSharpCorners, self.SHARPCORNERS)
	layout:AddChild(self.sharpCornersCheck)
end

function FA_AddPoint:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)

	self.autoWeldCheck:SetValue(FA_SharedUtils.autoWeld)
	self.autoFillCheck:SetValue(FA_SharedUtils.autoFill)
	self.sharpCornersCheck:SetValue(FA_SharedUtils.sharpCorners)
end

function FA_AddPoint:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.AUTOWELD) then
		FA_SharedUtils.autoWeld = self.autoWeldCheck:Value()
	elseif (msg == self.AUTOFILL) then
		FA_SharedUtils.autoFill = self.autoFillCheck:Value()
	elseif (msg == self.SHARPCORNERS) then
		FA_SharedUtils.sharpCorners = self.sharpCornersCheck:Value()
	elseif (msg == self.DELETE) then

		if (moho:CountSelectedPoints() > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			MOHO.DeleteSelectedPoints(mesh)
			view:DrawMe()
		end

	elseif (msg >= self.SELECTITEM) then
		FA_SharedUtils:SelectItem(moho, mesh, msg - self.SELECTITEM)
	end
end
