-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "FA_CreateShape"

-- **************************************************
-- General information about this script
-- **************************************************

FA_CreateShape = {}

function FA_CreateShape:Name()
	return "Create Shape"
end

function FA_CreateShape:Version()
	return "5.0"
end

function FA_CreateShape:Description()
	return "Select edges to define a shape (press <space> to create shape, <shift> to add selection)"
end

function FA_CreateShape:Creator()
	return "Lost Marble, Modified by Fazek"
end

function FA_CreateShape:UILabel()
	return("Create Shape")
end

function FA_CreateShape:LoadPrefs(prefs)
	FA_SharedUtils:Load_Prefs(prefs)
	self.autoFill = prefs:GetBool("FA_CreateShape.autoFill", false)
end

function FA_CreateShape:SavePrefs(prefs)
	FA_SharedUtils:Save_Prefs(prefs)
	prefs:SetBool("FA_CreateShape.autoFill", self.autoFill)
end

-- **************************************************
-- Recurring values
-- **************************************************

FA_CreateShape.autoFill = false

-- **************************************************
-- The guts of this script
-- **************************************************

function FA_CreateShape:IsEnabled(moho)

	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	if (moho:CountEdges() > 0) then
		return true
	end
	return false
end

function FA_CreateShape:OnMouseDown(moho, mouseEvent)
	FA_SharedUtils.selectionMode= 2
	FA_SharedUtils.floodOverflow= false
	FA_SharedUtils:MouseDown(moho, mouseEvent)
end

function FA_CreateShape:OnMouseMoved(moho, mouseEvent)
	FA_SharedUtils:MouseMoved(moho, mouseEvent)
end

function FA_CreateShape:OnMouseUp(moho, mouseEvent)
	FA_SharedUtils:MouseUp(moho, mouseEvent)

	if (FA_SharedUtils.floodOverflow) then
		LM.GUI.Alert(LM.GUI.ALERT_INFO,"The mesh is too complex to select.","Increase the value in the Flood-Select options.")
	elseif (self.autoFill) then
		local mesh = moho:Mesh()
		if (mesh == nil) then return end

		local view= mouseEvent.view

		if (moho:CountSelectedEdges() > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
	--filled shape?
			local shapeID = moho:CreateShape(true)

			if (shapeID >= 0) then
				moho:NewKeyframe(CHANNEL_FILL)
				moho:NewKeyframe(CHANNEL_LINE)
				moho:UpdateSelectedChannels()
			end
			view:DrawMe()
		end
	end
end

function FA_CreateShape:OnKeyDown(moho, keyEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then return end

	if (keyEvent.key == ' ') then
		self:HandleMessage(moho, keyEvent.view, self.CREATE)
	elseif (keyEvent.keyCode == LM.GUI.KEY_UP) then
		if (keyEvent.shiftKey) then -- raise shape to top
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:RaiseShape(i, true)
					break
				end
			end
		else -- raise shape
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:RaiseShape(i, false)
					break
				end
			end
		end
		keyEvent.view:DrawMe()
	elseif (keyEvent.keyCode == LM.GUI.KEY_DOWN) then
		if (keyEvent.shiftKey) then -- lower shape to bottom
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:LowerShape(i, true)
					break
				end
			end
		else -- lower shape
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:LowerShape(i, false)
					break
				end
			end
		end
		keyEvent.view:DrawMe()
	else
		FA_SharedUtils:KeyDown(moho, keyEvent)
	end
end

function FA_CreateShape:DrawMe(moho, view)
	FA_SharedUtils:Draw_Me(moho, view, 1)
end

-- **************************************************
-- Create shape dialog
-- **************************************************

local FA_CreateShapeDialog = {}

function FA_CreateShapeDialog:new()
	local dlg = LM.GUI.SimpleDialog("Flood-Select Options", FA_CreateShapeDialog)
	local l = dlg:GetLayout()

	l:PushV(LM.GUI.ALIGN_LEFT,1)
		l:AddChild(LM.GUI.StaticText("The selection has a search limit to avoid"), LM.GUI.ALIGN_LEFT)
		l:AddChild(LM.GUI.StaticText("freezing when clicking on complex meshes."), LM.GUI.ALIGN_LEFT)
	l:Pop()
	l:AddPadding()

	l:PushH(LM.GUI.ALIGN_CENTER)
		l:AddChild(LM.GUI.StaticText("Search limit:"))
		dlg.maxStep = LM.GUI.TextControl(0, "0000", 0, LM.GUI.FIELD_UINT)
		dlg.maxStep:SetWheelInc(10)
		l:AddChild(dlg.maxStep)
	l:Pop()

	l:AddChild(LM.GUI.Button(FA_SharedUtils.txReset, FA_CreateShape.RESET))
	return dlg
end

function FA_CreateShapeDialog:UpdateWidgets()
	self.maxStep:SetValue(FA_SharedUtils.floodMaxStep)
end

function FA_CreateShapeDialog:OnOK()
	FA_SharedUtils.floodMaxStep= self.maxStep:IntValue()
end

function FA_CreateShapeDialog:OnValidate()
	local b = true

	if (not self:Validate(self.maxStep, 20, 9999)) then
		b = false
	end
	return b
end

function FA_CreateShapeDialog:HandleMessage(msg)

	if (msg == FA_CreateShape.RESET) then
		self.maxStep:SetValue(200)
	end
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

FA_CreateShape.CREATE = MOHO.MSG_BASE
FA_CreateShape.OUTLINE = MOHO.MSG_BASE + 1
FA_CreateShape.AUTOFILL = MOHO.MSG_BASE + 2
FA_CreateShape.LASSO = MOHO.MSG_BASE + 3
FA_CreateShape.DUMMY = MOHO.MSG_BASE + 4
FA_CreateShape.RESET = MOHO.MSG_BASE + 5
FA_CreateShape.SELECTITEM = MOHO.MSG_BASE + 6

function FA_CreateShape:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu(FA_SharedUtils.txSelectGroup)

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	self.createBut = LM.GUI.Button(FA_SharedUtils.txCreateShape, self.CREATE)
	layout:AddChild(self.createBut)

	self.outlineBut = LM.GUI.Button("Outline only", self.OUTLINE)
	layout:AddChild(self.outlineBut)

	self.dlog = FA_CreateShapeDialog:new()
	self.dPopup = LM.GUI.PopupDialog("Flood-Select Options")
	self.dPopup:SetDialog(self.dlog)
	layout:AddChild(self.dPopup)

	self.autoFillCheck = LM.GUI.CheckBox(FA_SharedUtils.txAutoFill, self.AUTOFILL)
	layout:AddChild(self.autoFillCheck)

	self.lassoCheck = LM.GUI.CheckBox(FA_SharedUtils.txLasso, self.LASSO)
	layout:AddChild(self.lassoCheck)
end

function FA_CreateShape:UpdateWidgets(moho)
	self.autoFillCheck:SetValue(self.autoFill)
	self.lassoCheck:SetValue(FA_SharedUtils.lassoMode)

	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end
	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)
	self.createBut:Enable(moho:CountSelectedEdges() > 0)
end

function FA_CreateShape:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.CREATE or msg == self.OUTLINE) then

		if (moho:CountSelectedEdges() > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
	--filled shape?
			local shapeID= -1

			if (msg == self.CREATE) then
				shapeID = moho:CreateShape(true)
			end

			if (shapeID < 0) then
	--outline?
				shapeID = moho:CreateShape(false)
				if (shapeID >= 0) then
					mesh:SelectNone()
					mesh:Shape(shapeID).fFillAllowed = false
				end
			end
			if (shapeID >= 0) then
				mesh:SelectNone()
				mesh:Shape(shapeID).fSelected = true
				moho:NewKeyframe(CHANNEL_FILL)
				moho:NewKeyframe(CHANNEL_LINE)
				moho:UpdateSelectedChannels()
			end
			view:DrawMe()
		end

	elseif (msg == self.AUTOFILL) then
		self.autoFill = self.autoFillCheck:Value()
	elseif (msg == self.LASSO) then
		FA_SharedUtils.lassoMode = self.lassoCheck:Value()
	elseif (msg >= self.SELECTITEM) then
		FA_SharedUtils:SelectItem(moho, mesh, msg - self.SELECTITEM)
	end
end
