-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "FA_Polygon"

-- **************************************************
-- General information about this script
-- **************************************************

FA_Polygon = {}

function FA_Polygon:Name()
	return "Polygon"
end

function FA_Polygon:Version()
	return "5.0"
end

function FA_Polygon:Description()
	return "Draw a regular polygon (<ctrl> to constrain)"
end

function FA_Polygon:Creator()
	return "Lost Marble, Modified by Fazek"
end

function FA_Polygon:UILabel()
	return("Polygon")
end

function FA_Polygon:LoadPrefs(prefs)
	FA_SharedUtils:Load_Prefs(prefs)
	self.cornerPoints = prefs:GetInt("FA_Polygon.cornerPoints", 5)
	self.starFactor = prefs:GetFloat("FA_Polygon.starFactor", 1.0)
	self.starMode = prefs:GetBool("FA_Polygon.starMode", false)
end

function FA_Polygon:SavePrefs(prefs)
	FA_SharedUtils:Save_Prefs(prefs)
	prefs:SetInt("FA_Polygon.cornerPoints", self.cornerPoints)
	prefs:SetFloat("FA_Polygon.starFactor", self.starFactor)
	prefs:SetBool("FA_Polygon.starMode", self.starMode)
end

-- **************************************************
-- Recurring values
-- **************************************************

FA_Polygon.cornerPoints= 5
FA_Polygon.starFactor= 1.0
FA_Polygon.starMode= false
FA_Polygon.startVec= LM.Vector2:new_local()
FA_Polygon.startID= 0
FA_Polygon.endID= 0

-- **************************************************
-- The guts of this script
-- **************************************************

function FA_Polygon:IsEnabled(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return false
	end
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	return true
end

function FA_Polygon:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then return end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	local startID= mesh:CountPoints()
	mesh:SelectNone()

	self.startVec:Set(mouseEvent.vec)
	if (moho.gridOn) then
		moho:SnapToGrid(self.startVec)
	end

	local cpt= self.cornerPoints

	if (self.starMode) then
		cpt= cpt * 2
	end

	mesh:AddLonePoint(self.startVec, moho.frame)

	for i= 1, cpt do
		mesh:AppendPoint(self.startVec, moho.frame)
	end
	mesh:WeldPoints(startID + cpt, startID, 0)

	self.startID= startID
	self.endID= startID + cpt - 1

	for i= startID, self.endID do
		mesh:Point(i):SetCurvature(MOHO.PEAKED, moho.frame)
	end
	mesh:SelectConnected()

	if (FA_SharedUtils.autoFill) then
		local shapeID = moho:CreateShape(true)

		if (shapeID >= 0) then
			if (not FA_SharedUtils.autoOutline) then
				mesh:Shape(shapeID).fHasOutline = false
			end
		end
	elseif (FA_SharedUtils.autoOutline) then
		moho:CreateShape(false)
	end
	mouseEvent.view:DrawMe()
end

function FA_Polygon:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then return end

	local v1= mouseEvent.vec - self.startVec
	local v2= LM.Vector2:new_local()
	local angle= math.atan2(v1.y,v1.x)
	local angleStep= math.pi * 2 / self.cornerPoints
	v1:Rotate(-angle)

	if (moho.gridOn) then
		v1= v1 + self.startVec
		moho:SnapToGrid(v1)
		v1= v1 - self.startVec
	end

	if (mouseEvent.ctrlKey) then
		angle= angle / (math.pi / 4)
		angle= (math.pi / 4) * LM.Round(angle)
	end

	if (self.starMode) then
--draw a star
		local v3= LM.Vector2:new_local()
		v3:Set(v1)
		v3= v3 * math.cos(angleStep * self.starFactor) / math.cos(angleStep / 2)
		v3:Rotate(angleStep / 2)

		for i= self.startID, self.endID, 2 do
			v2:Set(v1)
			v2:Rotate(angle)
			mesh:Point(i).fPos:Set(v2 + self.startVec)
			v2:Set(v3)
			v2:Rotate(angle)
			mesh:Point(i+1).fPos:Set(v2 + self.startVec)
			angle= angle + angleStep
		end
	else
--draw a regular polygon

		for i= self.startID, self.endID do
			v2:Set(v1)
			v2:Rotate(angle)
			mesh:Point(i).fPos:Set(v2 + self.startVec)
			angle= angle + angleStep
		end
	end
	moho:AddPointKeyframe(moho.frame)
	mouseEvent.view:DrawMe()
end

function FA_Polygon:OnMouseUp(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then return end

	moho:AddPointKeyframe(moho.frame)
	mouseEvent.view:DrawMe()
	moho:NewKeyframe(CHANNEL_POINT)
	moho:NewKeyframe(CHANNEL_CURVE)
	moho:UpdateSelectedChannels()
end

function FA_Polygon:OnKeyDown(moho, keyEvent)
	FA_SharedUtils:KeyDown(moho, keyEvent)
end

-- **************************************************
-- Preview dialog
-- **************************************************

local FA_PolygonPreviewDialog = {}

function FA_PolygonPreviewDialog:new()
	local dlg = LM.GUI.SimpleDialog("Freehand Options", FA_PolygonPreviewDialog)
	local l = dlg:GetLayout()

	dlg.preview = MOHO.MeshPreview(100,100)
	l:AddChild(dlg.preview)

	l:AddChild(LM.GUI.Button("Close", LM.GUI.MSG_CANCEL))

	dlg.active= false
	return dlg
end

function FA_PolygonPreviewDialog:UpdateWidgets()
	self.active= true
	self:UpdatePreview()
end

function FA_PolygonPreviewDialog:HandleMessage()

	if (msg == LM.GUI.MSG_CANCEL) then
		self.active= false
	end
end

function FA_PolygonPreviewDialog:UpdatePreview()

	if (not self.active) then
		return
	end
	local mesh = self.preview:Mesh()
	mesh:Clear()

	local cpt= FA_Polygon.cornerPoints
	local angleStep= math.pi * 2 / cpt
	local v1= LM.Vector2:new_local()
	local v2= LM.Vector2:new_local()

	v1:Set(0,0.9)
	local angle= 0

	if (FA_Polygon.starMode) then
--draw a star
		local v3= LM.Vector2:new_local()
		v3:Set(v1)
		v3= v3 * math.cos(angleStep * FA_Polygon.starFactor) / math.cos(angleStep / 2)
		v3:Rotate(angleStep / 2)

		for i= 0, cpt do
			v2:Set(v1)
			v2:Rotate(angle)

			if (i == 0) then
				mesh:AddLonePoint(v2, 0)
			else
				mesh:AppendPoint(v2, 0)
			end
			v2:Set(v3)
			v2:Rotate(angle)
			mesh:AppendPoint(v2, 0)
			angle= angle + angleStep
		end
		cpt= cpt * 2
	else
--draw a regular polygon

		for i= 0, cpt do
			v2:Set(v1)
			v2:Rotate(angle)

			if (i == 0) then
				mesh:AddLonePoint(v2, 0)
			else
				mesh:AppendPoint(v2, 0)
			end
			angle= angle + angleStep
		end
	end
	mesh:WeldPoints(0, cpt, 0)

	for i= 0, cpt - 1 do
		mesh:Point(i):SetCurvature(MOHO.PEAKED, 0)
	end
	self.preview:CreateShape(false)
	self.preview:Refresh()
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

FA_Polygon.AUTOFILL = MOHO.MSG_BASE
FA_Polygon.AUTOOUTLINE = MOHO.MSG_BASE + 1
FA_Polygon.CORNERS = MOHO.MSG_BASE + 2
FA_Polygon.FACTOR = MOHO.MSG_BASE + 3
FA_Polygon.STAR = MOHO.MSG_BASE + 4
FA_Polygon.PREVIEW = MOHO.MSG_BASE + 5

function FA_Polygon:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText("Corners:"))
	self.textCorners= LM.GUI.TextControl(0, "000", self.CORNERS, LM.GUI.FIELD_UINT)
	self.textCorners:SetWheelInc(1)
	layout:AddChild(self.textCorners)

	layout:AddChild(LM.GUI.StaticText("Star factor:"))
	self.textFactor= LM.GUI.TextControl(0, "0.000", self.FACTOR, LM.GUI.FIELD_FLOAT)
	self.textFactor:SetWheelInc(0.125)
	layout:AddChild(self.textFactor)

	self.dlog = FA_PolygonPreviewDialog:new()
	self.popup= LM.GUI.PopupDialog("Preview...", true, self.PREVIEW)
	self.popup:SetDialog(self.dlog)
	layout:AddChild(self.popup)

	self.autoFillCheck = LM.GUI.CheckBox(FA_SharedUtils.txAutoFill, self.AUTOFILL)
	layout:AddChild(self.autoFillCheck)

	self.autoOutlineCheck = LM.GUI.CheckBox(FA_SharedUtils.txAutoOutline, self.AUTOOUTLINE)
	layout:AddChild(self.autoOutlineCheck)

	self.starCheck= LM.GUI.CheckBox("Draw star",self.STAR)
	layout:AddChild(self.starCheck)
end

function FA_Polygon:UpdateWidgets(moho)

	self.autoFillCheck:SetValue(FA_SharedUtils.autoFill)
	self.autoOutlineCheck:SetValue(FA_SharedUtils.autoOutline)

	if (self.cornerPoints < 3) then
		self.cornerPoints= 3
	end
	self.textCorners:SetValue(self.cornerPoints)

	if (self.starFactor < 0.5) then
		self.starFactor= 0.5
	elseif (self.starFactor > self.cornerPoints / 4) then
		self.starFactor= self.cornerPoints / 4
	end
	self.textFactor:SetValue(self.starFactor)
	self.textFactor:Enable(self.starMode)
	self.starCheck:SetValue(self.starMode)
end

function FA_Polygon:HandleMessage(moho, view, msg)
	if (msg == self.AUTOFILL) then
		FA_SharedUtils.autoFill = self.autoFillCheck:Value()
	elseif (msg == self.AUTOOUTLINE) then
		FA_SharedUtils.autoOutline = self.autoOutlineCheck:Value()
	elseif (msg == self.STAR) then
		self.starMode = self.starCheck:Value()
		self.textFactor:Enable(self.starMode)
		self.dlog:UpdatePreview()
	elseif (msg == self.CORNERS) then
		self.cornerPoints = self.textCorners:IntValue()

		if (self.cornerPoints < 3) then
			self.cornerPoints= 3
			self.textCorners:SetValue(self.cornerPoints)
		end

		if (self.starFactor > self.cornerPoints / 4) then
			self.starFactor= self.cornerPoints / 4
			self.textFactor:SetValue(self.starFactor)
		end
		self.dlog:UpdatePreview()
	elseif (msg == self.FACTOR) then
		self.starFactor = self.textFactor:FloatValue()

		if (self.starFactor < 0.5) then
			self.starFactor= 0.5
			self.textFactor:SetValue(self.starFactor)
		elseif (self.starFactor > self.cornerPoints / 4) then
			self.starFactor= self.cornerPoints / 4
			self.textFactor:SetValue(self.starFactor)
		end
		self.dlog:UpdatePreview()
	end
end
